@@ -41,6 +41,8 @@ class TestNereidProject(NereidTestCase):
         this method is called before each test function execution.
         """
         trytond.tests.test_tryton.install_module('nereid_project')
+        self.ActivityAllowedModel = POOL.get('nereid.activity.allowed_model')
+        self.Model = POOL.get('ir.model')
         self.Project = POOL.get('project.work')
         self.Company = POOL.get('company.company')
         self.Employee = POOL.get('company.employee')
@@ -200,11 +202,11 @@ class TestNereidProject(NereidTestCase):
             'localhost/project/project.jinja': '{{ project.name }}',
             'localhost/project/home.jinja': '{{ projects|length }}',
             'localhost/project/timesheet.jinja': '{{ employees|length }}',
-            'localhost/project/project-files.jinja':
+            'localhost/project/files.jinja':
                 '{{ project.children[0].attachments|length }}',
-            'localhost/project/project-permissions.jinja':
+            'localhost/project/permissions.jinja':
                 '{{ invitations|length }}',
-            'localhost/project/project-plan.jinja': '{{  }}',
+            'localhost/project/plan.jinja': '{{  }}',
             'localhost/project/compare-performance.jinja':
                 '{{ employees|length }}',
             'localhost/project/emails/text_content.jinja': '',
@@ -768,6 +770,12 @@ class TestNereidProject(NereidTestCase):
                 'parent': project.id,
                 'company': data['company'].id,
             })
+            task2 = self.Project.create({
+                'name': 'PQR_task',
+                'comment': 'task2',
+                'parent': project.id,
+                'company': data['company'].id,
+            })
 
             login_data = {
                 'email': 'email@example.com',
@@ -788,16 +796,79 @@ class TestNereidProject(NereidTestCase):
                 response = c.post(
                     '/en_US/attachment/-upload',
                     data={
-                        'file': (StringIO('testfile contents'), 'test.txt'),
+                        'file': (StringIO('testfile contents'), 'test1.txt'),
                         'task': task1.id,
                     },
                     content_type="multipart/form-data"
                 )
                 self.assertEqual(response.status_code, 302)
 
-                # File is added successfully
+                response = c.get('en_US/login')
+                self.assertTrue(
+                    u'Attachment added to ABC_task' in response.data
+                )
+
+                # File 'test1.txt' added successfully in task1
+                self.assertEqual(len(self.Attachment.search([])), 1)
+
+                # Add same file to other task
+                response = c.post(
+                    '/en_US/attachment/-upload',
+                    data={
+                        'file': (StringIO('testfile contents'), 'test1.txt'),
+                        'task': task2.id,
+                    },
+                    content_type="multipart/form-data"
+                )
+                self.assertEqual(response.status_code, 302)
+
+                response = c.get('en_US/login')
+                self.assertTrue(
+                    u'Attachment added to PQR_task' in response.data
+                )
+
+                # Same file 'test1.txt' added successfully in task2
+                self.assertEqual(len(self.Attachment.search([])), 2)
+
+                # Upload same file again
+                response = c.post(
+                    '/en_US/attachment/-upload',
+                    data={
+                        'file': (StringIO('testfile contents'), 'test1.txt'),
+                        'task': task1.id,
+                    },
+                    content_type="multipart/form-data"
+                )
+                self.assertEqual(response.status_code, 302)
+
+                # Check Flash Message
+                response = c.get('/en_US/login')
+                self.assertTrue(
+                    u'File already exists with same name, please choose ' +
+                    'another file or rename this file to upload !!' in
+                    response.data
+                )
+
+                # No same file added in same task
+                self.assertEqual(len(self.Attachment.search([])), 2)
+
+                # Add same file content with different file name
+                response = c.post(
+                    '/en_US/attachment/-upload',
+                    data={
+                        'file': (StringIO('testfile contents'), 'test2.txt'),
+                        'task': task1.id,
+                    },
+                    content_type="multipart/form-data"
+                )
+                self.assertEqual(response.status_code, 302)
+
+                # 2nd file with same content is added successfully
                 response = c.get('/en_US/project-%d/-files' % project.id)
-                self.assertEqual(response.data, '1')
+                self.assertEqual(response.data, '2')
+
+                # Total file added in attachments
+                self.assertEqual(len(self.Attachment.search([])), 3)
 
     def test_0130_render_files(self):
         """
