@@ -40,6 +40,8 @@ class TestTask(NereidTestCase):
         this method is called before each test function execution.
         """
         trytond.tests.test_tryton.install_module('nereid_project')
+        self.ActivityAllowedModel = POOL.get('nereid.activity.allowed_model')
+        self.Model = POOL.get('ir.model')
         self.Company = POOL.get('company.company')
         self.Employee = POOL.get('company.employee')
         self.Currency = POOL.get('currency.currency')
@@ -162,6 +164,11 @@ class TestTask(NereidTestCase):
             'color': 'color2',
             'project': project1.id
         })
+        tag3 = self.Tag.create({
+            'name': 'tag3',
+            'color': 'color3',
+            'project': project1.id
+        })
 
         # Nereid Permission
         permission = self.Permission.search([
@@ -190,6 +197,7 @@ class TestTask(NereidTestCase):
             'project1': project1,
             'tag1': tag1,
             'tag2': tag2,
+            'tag3': tag3,
         }
 
     def create_task_dafaults(self):
@@ -740,51 +748,53 @@ class TestTask(NereidTestCase):
                 # Login Success
                 self.assertEqual(response.status_code, 302)
                 self.assertEqual(response.location, 'http://localhost/en_US/')
+                with Transaction().set_context(
+                    {'company': data['company'].id}
+                ):
+                    # Mark time
+                    response = c.post(
+                        '/en_US/task-%d/-mark-time' % task.id,
+                        data={
+                            'hours': '8',
+                        }
+                    )
 
-                # Mark time
-                response = c.post(
-                    '/en_US/task-%d/-mark-time' % task.id,
-                    data={
-                        'hours': '8',
-                    }
-                )
+                    self.assertEqual(response.status_code, 302)
 
-                self.assertEqual(response.status_code, 302)
+                    # Check Flash Message
+                    response = c.get('/en_US/login')
+                    self.assertTrue(
+                        u'Time has been marked on task ABC_task' in
+                        response.data
+                    )
 
-                # Check Flash Message
-                response = c.get('/en_US/login')
-                self.assertTrue(
-                    u'Time has been marked on task ABC_task' in
-                    response.data
-                )
+                    # Logout
+                    response = c.get('/en_US/logout')
 
-                # Logout
-                response = c.get('/en_US/logout')
+                    # Login with other user
+                    response = c.post('/en_US/login', data=login_data2)
 
-                # Login with other user
-                response = c.post('/en_US/login', data=login_data2)
+                    # Login Success
+                    self.assertEqual(response.status_code, 302)
+                    self.assertEqual(response.location, 'http://localhost/en_US/')
 
-                # Login Success
-                self.assertEqual(response.status_code, 302)
-                self.assertEqual(response.location, 'http://localhost/en_US/')
+                    # Mark time when user is not employee
+                    response = c.post(
+                        '/en_US/task-%d/-mark-time' % task.id,
+                        data={
+                            'hours': '8',
+                        }
+                    )
 
-                # Mark time when user is not employee
-                response = c.post(
-                    '/en_US/task-%d/-mark-time' % task.id,
-                    data={
-                        'hours': '8',
-                    }
-                )
+                    self.assertEqual(response.status_code, 302)
+                    response = c.get('/en_US/logout')
 
-                self.assertEqual(response.status_code, 302)
-                response = c.get('/en_US/logout')
-
-                # Check Flash Message
-                response = c.get('/en_US/login')
-                self.assertTrue(
-                    u'Only employees can mark time on tasks!' in
-                    response.data
-                )
+                    # Check Flash Message
+                    response = c.get('/en_US/login')
+                    self.assertTrue(
+                        u'Only employees can mark time on tasks!' in
+                        response.data
+                    )
 
     def test_0130_change_estimated_hours(self):
         """
@@ -1042,6 +1052,69 @@ class TestTask(NereidTestCase):
                         2
                     )
 
+    def test_0200_create_task_with_multiple_tags(self):
+        """
+        Adding more than one tag to task which already exist in a project
+        """
+        with Transaction().start(DB_NAME, USER, CONTEXT):
+            data = self.create_defaults()
+            app = self.get_app(DEBUG=True)
+
+            login_data = {
+                'email': 'email@example.com',
+                'password': 'password',
+            }
+            with app.test_client() as c:
+                response = c.post('/en_US/login', data=login_data)
+                self.assertEqual(response.status_code, 302)
+
+                with Transaction().set_context(
+                    {'company': data['company'].id}
+                ):
+                    # No task created
+                    self.assertEqual(
+                        len(self.Project.search([('type', '=', 'task')])),
+                        0
+                    )
+
+                    # Create Task
+                    response = c.post(
+                        '/en_US/project-%d/task/-new' % data['project1'].id,
+                        data={
+                            'name': 'Task with multiple tags',
+                            'description': 'Multi selection tags field',
+                            'tags': [
+                                data['tag1'].id,
+                                data['tag2'].id,
+                                data['tag3'].id,
+                            ],
+                        }
+                    )
+                    self.assertEqual(response.status_code, 302)
+                    # One task created
+                    self.assertEqual(
+                        len(self.Project.search([('type', '=', 'task')])),
+                        1
+                    )
+                    self.assertTrue(
+                        self.Project.search([
+                            ('name', '=', 'Task with multiple tags')
+                        ])
+                    )
+
+                    task, = self.Project.search([
+                        ('name', '=', 'Task with multiple tags'),
+                    ])
+
+                    # Tags added in above created task
+                    self.assertEqual(len(task.tags), 3)
+
+                    response = c.get('/en_US/login')
+                    self.assertTrue(
+                        u'Task successfully added to project ABC' in
+                        response.data
+                    )
+
 
 def suite():
     "Nereid test suite"
